<?php

namespace Drupal\Tests\overview_builder\Functional;

use Drupal\Tests\BrowserTestBase;

/**
 * Test the overview builder views display extender.
 *
 * @group overview_builder
 */
class OverviewBuilderViewsAdminTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['overview_builder_paragraphs_test'];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->drupalLogin($this->rootUser);
  }

  /**
   * Test that exposed views are picked up in the overview builder field.
   */
  public function testExposedView() {
    $page = $this->getSession()->getPage();

    // Visit the overview page node, so the list of overview builder plugins is
    // cached.
    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();

    // Set the view exposed with no admin label.
    $this->drupalGet('/admin/structure/views/view/news');
    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'No');
    $this->click('#views-block-1-overview-builder');
    $this->submitForm([
      'exposed' => TRUE,
    ], 'Apply');

    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'Yes');
    $this->submitForm([], 'Save');

    $news_view_config = $this->config('views.view.news');
    $this->assertEquals([
      'exposed' => TRUE,
      'admin_label' => NULL,
    ], $news_view_config->get('display')['block_1']['display_options']['display_extenders']['overview_builder']);

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    // Check that only the exposed display is shown.
    $this->assertSession()->optionExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');
    $this->assertSession()->optionNotExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_2');
    $this->assertSession()->elementContains('css', '[name="field_paragraphs[0][subform][field_overview][0][plugin_id]"]', 'News - Block');

    // Set the view exposed with an admin label.
    $this->drupalGet('/admin/structure/views/view/news');
    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'Yes');
    $this->click('#views-block-1-overview-builder');
    $this->submitForm([
      'exposed' => TRUE,
      'admin_label' => 'News overview Admin label',
    ], 'Apply');

    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'Yes');
    $this->submitForm([], 'Save');

    $news_view_config = $this->config('views.view.news');
    $this->assertEquals([
      'exposed' => TRUE,
      'admin_label' => 'News overview Admin label',
    ], $news_view_config->get('display')['block_1']['display_options']['display_extenders']['overview_builder']);

    // Set the view no longer exposed.
    $this->drupalGet('/admin/structure/views/view/news');
    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'Yes');
    $this->click('#views-block-1-overview-builder');
    $this->submitForm([
      'exposed' => FALSE,
    ], 'Apply');

    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'No');
    $this->submitForm([], 'Save');

    $news_view_config = $this->config('views.view.news');
    $this->assertEquals([
      'exposed' => FALSE,
      'admin_label' => NULL,
    ], $news_view_config->get('display')['block_1']['display_options']['display_extenders']['overview_builder']);

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionNotExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');
  }

  /**
   * Test that a disabled display is not shown in the overview builder field.
   */
  public function testDisabledDisplay() {
    $page = $this->getSession()->getPage();
    $this->drupalGet('/admin/structure/views/view/news');

    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'No');
    $this->click('#views-block-1-overview-builder');
    $this->submitForm([
      'exposed' => TRUE,
    ], 'Apply');

    $this->submitForm([], 'Save');

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');

    $title = $this->randomString();
    $this->submitForm([
      'title[0][value]' => $title,
      'field_paragraphs[0][subform][field_overview][0][plugin_id]' => 'views_overview:news__block_1',
    ], 'Save');
    $this->assertSession()->elementExists('css', 'main .views-element-container');

    $this->drupalGet('/admin/structure/views/view/news');
    $this->submitForm([], 'Disable Block');
    $this->submitForm([], 'Save');

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionNotExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');

    $node = $this->drupalGetNodeByTitle($title);
    $this->drupalGet($node->toUrl());
    $this->assertSession()->elementNotExists('css', 'main .views-element-container');
  }

  /**
   * Test that a disabled view is not shown in the overview builder field.
   */
  public function testDisabledView() {
    $page = $this->getSession()->getPage();
    $this->drupalGet('/admin/structure/views/view/news');

    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'No');
    $this->click('#views-block-1-overview-builder');
    $this->submitForm([
      'exposed' => TRUE,
    ], 'Apply');

    $this->submitForm([], 'Save');

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');

    $title = $this->randomString();
    $this->submitForm([
      'title[0][value]' => $title,
      'field_paragraphs[0][subform][field_overview][0][plugin_id]' => 'views_overview:news__block_1',
    ], 'Save');
    $this->assertSession()->elementExists('css', 'main .views-element-container');

    $this->drupalGet('/admin/structure/views');
    $this->clickLink('Disable', 3);

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionNotExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');

    $node = $this->drupalGetNodeByTitle($title);
    $this->drupalGet($node->toUrl());
    $this->assertSession()->elementNotExists('css', 'main .views-element-container');
  }

  /**
   * Test that a deleted view is no longer shown in the overview builder field.
   */
  public function testDeletedView() {
    $page = $this->getSession()->getPage();
    $this->drupalGet('/admin/structure/views/view/news');

    $this->assertSession()->elementTextContains('css', '#views-block-1-overview-builder', 'No');
    $this->click('#views-block-1-overview-builder');
    $this->submitForm([
      'exposed' => TRUE,
    ], 'Apply');

    $this->submitForm([], 'Save');

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');

    $title = $this->randomString();
    $this->submitForm([
      'title[0][value]' => $title,
      'field_paragraphs[0][subform][field_overview][0][plugin_id]' => 'views_overview:news__block_1',
    ], 'Save');
    $this->assertSession()->elementExists('css', 'main .views-element-container');

    $this->drupalGet('/admin/structure/views/view/news');
    $this->clickLink('Delete view');
    $this->submitForm([], 'Delete');

    $this->drupalGet('/node/add/overview_page');
    $page->findButton('field_paragraphs_add_more')->press();
    $this->assertSession()->optionNotExists('field_paragraphs[0][subform][field_overview][0][plugin_id]', 'views_overview:news__block_1');

    $node = $this->drupalGetNodeByTitle($title);
    $this->drupalGet($node->toUrl());
    $this->assertSession()->elementNotExists('css', 'main .views-element-container');
  }

}
